<?php if (!defined('PmWiki')) exit();

/*

 PmWiki XML-RPC Interface Manager

 -- Copyright --

 Copyright 2005 by James McDuffie (ttq3sxl02@sneakemail.com)

 -- License --

 GNU GPL

 -- Description --

 This class manages the different XML-RPC interfaces that can be used with
 PmWiki. It looks at configuration variables to know which interfaces to
 load into the XML-RPC server.

 See: http://www.pmwiki.org/wiki/Cookbook/XMLRPC for more information.
   
 -- Installation Instructions --

 * Add the following line to include the xmlrpc.php script in your config.php

     include_once('cookbook/xmlrpc.php');

 -- Configuration --

 By default all interfaces are enabled. They can be turned off by setting the
 corresponding variable named below to 0 in your config.php before xmlrpc.php
 is included

 Interface usage varibles:
   $UseBloggerInterface
   $UseVoodooPadInterface

 For instance adding:
   $UseBloggerInterface = 0;
 Will disable usage of the Blogger interface. 

 -- History --

 Version 0.1 
 * Initial version

 Version 0.2
 * In blogger.php, fixed variable names in PostWikiPage that would cause postid to be returned when editing a page when it should be returning only a boolean
 * Upgraded to CVS version of xmlrpc.in and xmlrpcs.inc
 * Commented out line of code in xmlrpc.inc that was causing dumplicate &xxx;
 values in Php version 5+
 * Changed xmlrpc.inc so that tabs, newlines and line feeds will not be
 converted into special characters
 * Minor changes to voodoopad.php related to pagename keys with spaces

*/

define(PMWIKI_XMLRPC, '0.2'); 

// Configuration variables
SDV($UseBloggerInterface, 1);
SDV($UseVoodooPadInterface, 1);

// Whenever ?action=xmlrpc is called this method is used
$HandleActions['xmlrpc'] = 'HandleXMLRPC';

// Only include the other XML RPC code when needed for the xmlrpc action
if($action == "xmlrpc") {
  require_once("xmlrpc/xmlrpc.inc");
  require_once("xmlrpc/xmlrpcs.inc");
}

function HandleXMLRPC ($pagename) {
  global $UseBloggerInterface, $UseVoodooPadInterface;

  $FunctionMap = Array();

  if($UseBloggerInterface) {
    require_once("xmlrpc/blogger.php");
    $FunctionMap = array_merge($FunctionMap, $BloggerFuncMap);
  }

  if($UseVoodooPadInterface) {
    require_once("xmlrpc/voodoopad.php");
    $FunctionMap = array_merge($FunctionMap, $VoodooPadFuncMap);
  }

  $s = new xmlrpc_server( $FunctionMap );
}

/******************************************************************
  Helper Functions For All Interfaces
 ******************************************************************/

// Sets up the user credential for further checking as well as
// checks for anything that can be checked just based on a username 
// and password
function ValidateUser ($username, $password) {
  global $Author;
  
  // Set author as username if it has not already been set
  SDV($Author, $username);

  // Set the variable that PmWiki's basic authentification will check
  $_POST['authpw'] = $password;

  // If using UserAuth then
  // Set variables for further use by UserAuth then check for a
  // valid username/password by seeing if the DefaultPage can be accessed
  if (defined('USER_AUTH_VERSION') && constant('USER_AUTH_VERSION') >= 0.6) {
    global $UserInstanceVars, $DefaultPage;

    $UserInstanceVars->SetInstanceUsername($username);
    $UserInstanceVars->SetInstancePassword($password);

    return CheckUserAuthPassword( $username, $password, 
				  "read", $DefaultPage, false );
  }

  return true;
}

?>
