<?php if (!defined('PmWiki')) exit();

/*
 
 PmWiki Blogger XML-RPC Interface

 -- Copyright --

 Copyright 2005 by James McDuffie (ttq3sxl02@sneakemail.com)

 -- License --

 GNU GPL

 -- Description --

 Provides an interface between PmWiki and any client using the Blogger
 XML-RPC api.

 -- History --

 Version 0.1 
 * Initial version

 Version 0.11
 * Fixed variable names in PostWikiPage that would cause postid to be returned
 when editing a page when it should be returning only a boolean

*/

define(PMWIKI_XMLRPC_BLOGGER, '0.11'); 

$BloggerFuncMap = 
  array 
  (
   "blogger.newPost" => 
   array( "function" => array("BloggerXMLRPC", "NewPage")),

   "blogger.getUserInfo" => 
   array( "function" => array("BloggerXMLRPC","GetUserInfo")),

   "blogger.getPost" =>
   array( "function" => array("BloggerXMLRPC","GetPage")),

   "blogger.editPost" => 
   array( "function" => array("BloggerXMLRPC","EditPage")),

   "blogger.deletePost" =>
   array( "function" => array("BloggerXMLRPC","DeletePage")),

   "blogger.getRecentPosts" =>
   array( "function" => array("BloggerXMLRPC","GetRecentPages")),

   "blogger.getUserInfo" =>
   array( "function" => array("BloggerXMLRPC","GetUserInfo")),

   "blogger.getUsersBlogs" =>
   array( "function" => array("BloggerXMLRPC", "GetAccessibleGroups")),
   );

// A class full of static methods is used to reduce namespace pollution
// Different XMLRPC interfaces might have similar method names
class BloggerXMLRPC {
  /******************************************************************
   Blogger Protocol Interface Functions
  ******************************************************************/

  // Return with user info which amounts to just the username/author
  function GetUserInfo($params) {
    $appkeyp=$params->getParam(0); $appkey=$appkeyp->scalarval();
    $usernamep=$params->getParam(1); $username=$usernamep->scalarval();
    $passwordp=$params->getParam(2); $password=$passwordp->scalarval();

    if(!ValidateUser($username,$password)) {
      return new xmlrpcresp(0, 101, "Invalid username or password");
    }
  
    $myStruct = 
      new xmlrpcval(array( "nickname" => new xmlrpcval($username),
			   "firstname" => new xmlrpcval("none"),
			   "lastname" => new xmlrpcval("none"),
			   "email" => new xmlrpcval("none"),
			   "userid" => new xmlrpcval($username),
			   "url" => new xmlrpcval("none")
			   ),"struct");
    return new xmlrpcresp($myStruct);
  }
 
  /* Posts a new submission to the CMS */
  function NewPage($params) {
    global $GroupPattern, $NamePattern;

    $appkeyp=$params->getParam(0); $appkey=$appkeyp->scalarval();
    $blogidp=$params->getParam(1); $groupname=$blogidp->scalarval();
    $usernamep=$params->getParam(2); $username=$usernamep->scalarval();
    $passwordp=$params->getParam(3); $password=$passwordp->scalarval();
    $passp=$params->getParam(4); $content=$passp->scalarval();
    $passp=$params->getParam(5); $publish=$passp->scalarval();
  
    if(!ValidateUser($username,$password)) {
      return new xmlrpcresp(0, 101, "Invalid username or password");
    }
  
    // Extract title of post, won't use it since the id passsed in params
    // identifies page
    ereg("<title>(.*)</title>",$content, $title);
    $title = $title[1];
    $content = ereg_replace("<title>(.*)</title>","",$content);

    if(empty($title)) {
      return new xmlrpcresp(0, 101, "Title required for determining pagename");  
    }

    if(preg_match("/^($GroupPattern)[\\/.]($NamePattern)\$/", $title)) {
      $pagename = $title;
    } else {
      $pagename = $groupname . "." . $title;
    }

    return BloggerXMLRPC::PostWikiPage($pagename, $content, $publish, true);
  }

  // Edits an existing PmWiki page
  function EditPage($params) {
    $appkeyp=$params->getParam(0); $appkey=$appkeyp->scalarval();
    $blogidp=$params->getParam(1); $pagename=$blogidp->scalarval();
    $usernamep=$params->getParam(2); $username=$usernamep->scalarval();
    $passwordp=$params->getParam(3); $password=$passwordp->scalarval();
    $passp=$params->getParam(4); $content=$passp->scalarval();
    $passp=$params->getParam(5); $publish=$passp->scalarval();

    if(!ValidateUser($username,$password)) {
      return new xmlrpcresp(0, 101, "Invalid username or password");
    }
  
    // Extract title of post, won't use it since the id passsed in params
    // identifies page
    ereg("<title>(.*)</title>",$content, $title);
    $title = $title[1];
    $content = ereg_replace("<title>(.*)</title>","",$content);
 
    // Now get the post information
    if(!PageExists($pagename)) {
      return new xmlrpcresp(0, 101, "Page not found");
    }
  
    return BloggerXMLRPC::PostWikiPage($pagename, $content, $publish);
  }

  // Deletes a specific PmWiki page
  function DeletePage($params) {
    $appkeyp=$params->getParam(0); $appkey=$appkeyp->scalarval();
    $blogidp=$params->getParam(1); $pagename=$blogidp->scalarval();
    $usernamep=$params->getParam(2); $username=$usernamep->scalarval();
    $passwordp=$params->getParam(3); $password=$passwordp->scalarval();
    $passp=$params->getParam(4); $publish=$passp->scalarval();

    if(!ValidateUser($username,$password)) {
      return new xmlrpcresp(0, 101, "Invalid username or password");
    }
  
    return new xmlrpcresp(0, 101, "Deletion of pages not supported.");

    // If implemented then upon success this should be returned
    //return new xmlrpcresp(new xmlrpcval(1,"boolean"));
  }

  // Get a specific PmWiki page
  function GetPage($params) {
    $appkeyp=$params->getParam(0); $appkey=$appkeyp->scalarval();
    $blogidp=$params->getParam(1); $pagename=$blogidp->scalarval();
    $usernamep=$params->getParam(2); $username=$usernamep->scalarval();
    $passwordp=$params->getParam(3); $password=$passwordp->scalarval();

    if(!ValidateUser($username,$password)) {
      return new xmlrpcresp(0, 101, "Invalid username or password");
    }

    if(!PageExists($pagename)) {
      return new xmlrpcresp(0, 101, "Page not found");
    }

    $page = RetrieveAuthPage($pagename, "read", false);
    
    if(!$page) {
      return new xmlrpcresp(0, 101, "Permission denied");
    }

    $myStruct = BloggerXMLRPC::PageAsXMLStruct($page);
  
    return new xmlrpcresp($myStruct);
  }

  // Retrieves a certain number of PmWiki pages ordered by date
  function GetRecentPages($params) {
    $appkeyp=$params->getParam(0); $appkey=$appkeyp->scalarval();
    $blogidp=$params->getParam(1); $groupname=$blogidp->scalarval();
    $usernamep=$params->getParam(2); $username=$usernamep->scalarval();
    $passwordp=$params->getParam(3); $password=$passwordp->scalarval();
    $passp=$params->getParam(4); $number=$passp->scalarval();

    if(!ValidateUser($username,$password)) {
      return new xmlrpcresp(0, 101, "Invalid username or password");
    }

    // Gather pages in an array for sorting by time
    $grouppages = Array();
    $pagelist = ListPages();
    foreach($pagelist as $pagename) {
      $currgroup = FmtPageName('$Group', $pagename);

      // Skip pages not in the current requested group
      if($currgroup !== $groupname) continue;

      $page = RetrieveAuthPage($pagename, "read", false);

      // Skip pages we do not have access to
      if(!$page) continue;

      $grouppages[] = $page;
    }

    function ComparePageTime($a, $b) {
      if ($a['time'] > $b['time']) return -1;
      if ($a['time'] < $b['time']) return 1;
      return 0;
    }
    usort($grouppages, "ComparePageTime"); 

    $arrayval = Array();
    $pagenum = 0;
    foreach($grouppages as $page) {
      // Only return as many pages as are requested
      if($pagenum++ >= $number) break;

      $arrayval[]= BloggerXMLRPC::PageAsXMLStruct($page);
    }
   
    $myVal=new xmlrpcval($arrayval, "array");
    return new xmlrpcresp($myVal);
  }

  // Get the names of PmWiki Groups that are accessible to the user
  // as if these were the names of blogs. 
  function GetAccessibleGroups($params) {
    global $AuthFunction, $ScriptUrl;

    $appkeyp=$params->getParam(0); $appkey=$appkeyp->scalarval();
    $usernamep=$params->getParam(1); $username=$usernamep->scalarval();
    $passwordp=$params->getParam(2); $password=$passwordp->scalarval();

    // Authenticate user using parameters passed to method
    if(!ValidateUser($username, $password)) {
      return new xmlrpcresp(0, 101, "Invalid username or password");
    }

    $pagelist = ListPages();
    $arrayVal=Array();

    // Gather group names that are accessible to the user
    $grouplist = Array();
    foreach($pagelist as $pagename) {
      // Make sure an AuthFunction has been defined before trying to
      // use it
      if(isset($AuthFunction)) {
	$accessible = $AuthFunction($pagename, 'read', false);
      } else {
	$accessible = true;
      }
   
      if($accessible) {
	$groupname = FmtPageName('$Group',$pagename);
	$grouplist[$groupname]++;
      }
    }

    if(count($grouplist) == 0) {
      return new xmlrpcresp(0, 101, "No groups accessible to user");
    }
  
    foreach(array_keys($grouplist) as $groupname) {
      $arrayVal[] =
	new xmlrpcval(array("blogName" => new xmlrpcval($groupname),
			    "url" => new xmlrpcval($ScriptUrl."?n=".$groupname),
			    "blogid" => new xmlrpcval($groupname)),
		      "struct");
    }

    $myVal = new xmlrpcval($arrayVal, "array");

    return new xmlrpcresp($myVal);
  }

  /******************************************************************
   Blogger Protocol Helper Functions
  ******************************************************************/

  // Helper function that returns a PmWiki page as a XML structure
  function PageAsXMLStruct($page) {

    // Change Unix time into ISO8601 format
    $lastmodified = strftime("%Y-%m-%dT%H:%M:%S%z",$page['time']);

    return 
      new xmlrpcval(
		    array("userid" => new xmlrpcval($page['author']),
			  "dateCreated" => new xmlrpcval($lastmodified, "dateTime.iso8601"),
			  "content" => new xmlrpcval("<title>" . $page['name'] . "</title>" . $page['text']),
			  "postid" => new xmlrpcval($page['name'])
			  ),"struct");
  }

  function PostWikiPage($pagename, $content, $publish, $isnew=false) {
    global $EditFunctions;

    Lock(2);
    $oldpage = RetrieveAuthPage($pagename, 'edit', false);
    if (!$oldpage) {
      return new xmlrpcresp(0, 101, "Permission denied to edit page");
    }

    // If the publish flag is set then tell PmWiki that we are posting
    if($publish) $_POST['post'] = 1;

    PCache($pagename,$oldpage);

    // Set new content and perform all edit functions that apply on new page
    $newpage = $oldpage;
    $newpage['text'] = str_replace("\r",'', $content);
    foreach((array)$EditFunctions as $fn) $fn($pagename,$oldpage,$newpage);

    if($isnew) {
      return new xmlrpcresp(new xmlrpcval($pagename));
    } else {
      return new xmlrpcresp(new xmlrpcval(1,"boolean"));
    }
  }

} // End of BloggerXMLRPC Class

?>
